DeclareModule DS18B20

  ;-DeclareModule

  Enumeration
    #DS_Err_OK   ;  .
    #DS_Err_Port ;    ( OpenPort() -   ).
    #DS_Err_NoSensor ;   .
    #DS_Err_IO   ;      .
    #DS_Err_BaudRate ;     .
  EndEnumeration
  
  Structure DS_DataByte ;    
    L_Termo.a       ;   
    H_Termo.a       ;   
    H_User.a        ;   
    L_User.a        ;   
    Config.a        ;  
    x1.a            ;  (   )
    x2.a            ;  (   )
    x3.a            ;  (   )
    CRC.a           ;  
  EndStructure
  
  #PortOnly = -10
  
  Declare GetLastErr()
  Declare.s GetLastErr_String(Err=-1)
  Declare ClosePort(Port=#PortOnly)
  Declare OpenPort(ComPort.s, Port=#PortOnly)
  Declare ResetDS(Port=#PortOnly)
  Declare.a ByteRW(Byte.a=$FF, Port=#PortOnly)
  Declare.a GetCRC(*Buff, Count)
  Declare GetThermo(*Info.DS_DataByte, Port=#PortOnly)
  
EndDeclareModule

Module DS18B20

EnableExplicit

CompilerIf Defined(DCB, #PB_Structure)=0
  Structure DCB
    DCBlength.l
    BaudRate.l
    fbits.l
    wReserved.w
    XonLim.w
    XoffLim.w
    ByteSize.b
    Parity.b
    StopBits.b
    XonChar.b
    XoffChar.b
    ErrorChar.b
    EofChar.b
    EvtChar.b
    wReserved1.w
  EndStructure
CompilerEndIf

CompilerIf Defined(GetCommState_, #PB_Function)=0
  Import "kernel32.lib"
    GetCommState(PortID, *dcb.DCB)
    SetCommState(PortID, *dcb.DCB)
  EndImport
  
  Macro GetCommState_(PortID, dcb)
    GetCommState(PortID, dcb)
  EndMacro
  
  Macro SetCommState_(PortID, dcb)
    SetCommState(PortID, dcb)
  EndMacro
  
CompilerEndIf

;- Private

Structure CRC_Buff
  Byte.a[0]
EndStructure

Threaded tLastErr=0
Global gCurrent_ComPort_ID=-1

Macro TestPortID(pID)
  If pID=#PortOnly
    pID=gCurrent_ComPort_ID
  EndIf
EndMacro

Macro Odd(xx)
  ((xx) & 1)
EndMacro

Procedure ClearInBuf(Port) ;   .
  Protected Count
  
  Count = AvailableSerialPortInput(Port)
  If Count>0
    Protected Dim Buff.a(Count+2)
    ReadSerialPortData(Port, @Buff(), Count)
  EndIf
EndProcedure

Procedure SetBaudRate(Port, Rate)
  Protected Result=#False

  If IsSerialPort(Port)
    
    Protected dcb.DCB, PortID
    PortID = SerialPortID(Port)
    If PortID
      If GetCommState_(PortID, @dcb)  
        dcb\BaudRate  = Rate ;  .
        dcb\fbits | %101000000010001
        If SetCommState_(PortID, @dcb)
          Result = #True
        EndIf
      EndIf
    EndIf
    
    If Result=#False : tLastErr=#DS_Err_BaudRate : EndIf
  EndIf
    
  ProcedureReturn Result
EndProcedure


;-Public

Procedure GetLastErr()
  ProcedureReturn tLastErr
EndProcedure

Procedure.s GetLastErr_String(Err=-1)
  Protected String.s
  
  If Err=-1
    Err=tLastErr
  EndIf
  Select Err
    Case #DS_Err_OK : String = " "
    Case #DS_Err_Port : String = "   "
    Case #DS_Err_NoSensor : String = "   "
    Case #DS_Err_IO : String = "    "
    Case #DS_Err_BaudRate : String = "   "
  EndSelect
    
  ProcedureReturn String
EndProcedure

Procedure ClosePort(Port=#PortOnly)
  Protected Result = #False
  
  TestPortID(Port)
  
  If IsSerialPort(Port)
    SetSerialPortStatus(Port, #PB_SerialPort_DTR, 0)
    SetSerialPortStatus(Port, #PB_SerialPort_RTS, 0)
    CloseSerialPort(Port)
    Result = #True
    tLastErr = #DS_Err_OK
  Else
    tLastErr = #DS_Err_Port
  EndIf
  
  ProcedureReturn Result
EndProcedure

Procedure OpenPort(ComPort.s, Port=#PortOnly)
  Protected ID, PortID
  
  If Port>=0
    ClosePort(Port)
    PortID=Port
  Else
    PortID=#PB_Any
  EndIf
  
  ID = OpenSerialPort(PortID, ComPort, 115200, #PB_SerialPort_NoParity,
                      8, 1, #PB_SerialPort_NoHandshake, 24, 24)
  If ID
    If Port=#PortOnly
      gCurrent_ComPort_ID = ID
    ElseIf Port>=0
      ID=Port
    EndIf
    SetSerialPortStatus(ID, #PB_SerialPort_DTR, 1)
    SetSerialPortStatus(ID, #PB_SerialPort_RTS, 1)
    tLastErr = #DS_Err_OK
  Else
    tLastErr=#DS_Err_Port
  EndIf
  
  ProcedureReturn ID
EndProcedure

Procedure ResetDS(Port=#PortOnly) ;   ""  .
  Protected x, Result=#False
  
  TestPortID(Port)
  
  tLastErr=#DS_Err_OK
  
  If IsSerialPort(Port)
    If SetBaudRate(Port, 9600)
      
      ClearInBuf(Port)
      
      x=$F0
      WriteSerialPortData(Port, @x, 1)
      x = ElapsedMilliseconds()+100
      Repeat
        Delay(2)
      Until AvailableSerialPortInput(Port)>0 Or ElapsedMilliseconds()>x
      
      If AvailableSerialPortInput(Port)=1
        x=0
        If ReadSerialPortData(Port, @x, 1) = 1
          Result=Bool(x<>$F0)
          If Result=#False : tLastErr=#DS_Err_NoSensor : EndIf
        Else
          tLastErr=#DS_Err_NoSensor
        EndIf
      Else
        tLastErr=#DS_Err_NoSensor
      EndIf
      
      SetBaudRate(Port, 115200)
      
    EndIf
  Else
    tLastErr=#DS_Err_Port
  EndIf
  
  ProcedureReturn Result
EndProcedure

Procedure.a ByteRW(Byte.a=$FF, Port=#PortOnly)
  Protected Result.a, CRC.a, x.a, i, Time
  
 
  TestPortID(Port)
  
  tLastErr=#DS_Err_OK
  Result=$FF
  
  If IsSerialPort(Port)
    ClearInBuf(Port)
    
    Time=ElapsedMilliseconds()+100
    
    For i= 1 To 8
      
      x=Odd(Byte)*$FF
      WriteSerialPortData(Port, @x, 1)
      Byte>>1
      
      Repeat : Until AvailableSerialPortInput(Port)>0 Or ElapsedMilliseconds()>Time
      
      If AvailableSerialPortInput(Port)=1
        x=0
        If ReadSerialPortData(Port, @x, 1) = 1
          Result>>1
          If Odd(x) : Result | $80 : EndIf
        Else
          Result = $FF
          tLastErr=#DS_Err_IO
          Break
        EndIf
      Else
        Result = $FF
        tLastErr=#DS_Err_IO
        Break
      EndIf
      
    Next i
    
  Else
    tLastErr=#DS_Err_Port
  EndIf

  ProcedureReturn Result
EndProcedure

Procedure.a GetCRC(*Buff.CRC_Buff, Count)
  Protected CRC.a, Byte.a, i, x
  
  Count-1
  For i=0 To Count
    Byte=*Buff\Byte[i]
    For x=1 To 8
      If (Byte ! CRC) & 1
        CRC = ((CRC!$18)>>1)|$80
      Else
        CRC >> 1
      EndIf
      Byte>>1
    Next x
  Next i
  
  ProcedureReturn CRC
EndProcedure

Procedure GetThermo(*Info.CRC_Buff, Port=#PortOnly)
  Protected i, Result=#False
  
  tLastErr=#DS_Err_OK
  
  TestPortID(Port)
  If IsSerialPort(Port)
    ClearInBuf(Port)
    
    If ResetDS(Port)
      ByteRW($CC, Port)
      ByteRW($44, Port)
      Delay(800)
      If ResetDS(Port)
        ByteRW($CC, Port)
        ByteRW($BE, Port)
        
        For i=0 To 8
          *Info\Byte[i] = ByteRW($FF, Port)
        Next
        
        Result=#True
        
      EndIf
    EndIf
    
  Else
    tLastErr=#DS_Err_Port
  EndIf
    
  ProcedureReturn Result
EndProcedure

EndModule
; IDE Options = PureBasic 5.21 LTS (Windows - x86)
; CursorPosition = 68
; FirstLine = 46
; Folding = ---
; EnableXP